
# q-expansion of newform 5408.2.a.c, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5408
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-2], [-1], [0], [4], [0], [3], [-2], [-2], [5], [2], [-5], [-3], [4], [6], [13], [12], [-7], [-14], [6], [-7], [-8], [4], [-14], [2], [-11], [-14], [-6], [10], [-9], [22], [18], [1], [16], [-21], [22], [5], [16], [-4], [-2], [-16], [5], [-6], [21], [18], [14], [-18], [18], [18], [18], [6], [-12], [-7], [22], [11], [-24], [14], [12], [1], [21], [20], [19], [26], [-10], [-34], [3], [-8], [-1], [-10], [10], [-3], [20], [22], [-3], [2], [4], [33], [34], [33], [9], [-14], [-9], [-12], [-1], [-8], [4], [10], [25], [-21], [-20], [-24], [6], [32], [16], [-14], [28], [3], [-25], [-16], [23], [-22], [23], [-24], [6], [20], [29], [16], [1], [30], [-9], [34], [-1], [37], [-36], [20], [-17], [-4], [-38], [14], [-28], [-13], [-5], [26], [-38], [20], [-10], [-21], [48], [28], [-1], [20], [-48], [20], [38], [-38], [-38], [6], [22], [30], [-25], [-2], [-30], [-52], [0], [-31], [-26], [39], [-21], [-50], [24], [-41], [-29], [58], [56], [16], [32], [30], [-3], [55], [46], [-38], [42], [-38], [-18], [17], [18], [2], [-55], [-18], [53], [34], [3], [2], [-19], [46], [-9], [2], [-17], [0], [-34], [-34], [4], [1], [6], [-16], [29], [-42], [-26], [-19], [-60], [39], [-10], [-24], [18], [8], [-2], [33], [-51], [-11], [-38], [15], [30], [63], [-37], [-2], [57], [-20], [16], [21], [-28], [3], [14], [46], [-10], [66], [49], [-62], [15], [48], [-2], [-39], [70], [-55], [-8], [-16], [29], [-9], [16], [-18], [-44], [-11], [44], [-22], [15], [-6], [30], [30], [51], [-54], [-12], [-26], [70], [-28], [-69], [-51], [-12], [-30], [-48], [4], [-6], [62], [66], [-64], [59], [-42], [8], [62], [0], [-10], [65], [-32], [12], [23], [-27], [33], [2], [-46], [-39], [66], [-11], [13], [64], [-51], [-2], [-34], [14], [44], [-9], [53], [58], [16], [54], [72], [71], [32], [-22], [15], [-41], [44], [-81], [18], [72], [33], [26], [-46], [-3], [-62], [13], [-16], [38], [3], [-45], [60], [36], [30], [-11], [72], [-47], [-84], [-23], [-40], [-74], [43], [-16], [-42], [67], [-26], [-86], [2], [-17], [-66], [-30], [29], [56], [82], [43], [-14], [80], [-52], [-39], [-1], [57], [-66], [46], [-40], [58], [-85], [-34], [-39], [28], [-94], [-21], [-25], [92], [-54], [-32], [-43], [36], [30], [6], [-8], [2], [-53], [-58], [61], [71], [36], [-32], [-57], [18], [71], [3], [-30], [-88], [16], [-26], [10], [52], [19], [-78], [-70], [30], [73], [22], [-55], [54], [60], [-14], [47], [51], [-62], [-55], [44], [42], [84], [54], [0], [-74], [-66], [22], [-51], [-73], [72], [-20], [84], [62], [-56], [-58], [0], [-9], [34], [94], [20], [33], [39], [-34], [66], [-63], [4], [52], [-65], [5], [-60], [70], [-17], [-27], [58], [-42], [-54], [24], [-63], [103], [-8], [58], [-38], [7], [-32], [-103], [-58], [50]]
